#include "pins.h"

; data section

		.section .data
		.global display_frame
		.global display_row

display_frame:
      .byte 1
display_row:
      .byte 0

; bss section

      .section .bss
      .global display_buffer

display_buffer:
      .space 300

; text section

      .section .text
      .global display_init
      .global TIMER0_OVF_vect

display_init:
      push  r16

      ldi   r16, _BV(PINA_LATCH)|_BV(PINA_CLOCK)|_BV(PINA_DATA)
      out   _SFR_IO_ADDR(DDRA), r16

      ldi   r16, _BV(CS01)
      out   _SFR_IO_ADDR(TCCR0B), r16

      ldi   r16, _BV(TOIE0)
      sts   _SFR_MEM_ADDR(TIMSK0), r16

      pop   r16

      ret

TIMER0_OVF_vect:
		push	r0
      push  r1
      push  r16
      push  r17
      push  r18
      push  r19
      push  r20
      push  r28
      push  r29

      in    r20, _SFR_IO_ADDR(SREG)

		lds   r16, display_frame         ; r16 = frame number
      lds   r17, display_row           ; r17 = row number

		ldi   r28, lo8(display_buffer)   ; r29:r28 (Y) = row base pointer
		ldi   r29, hi8(display_buffer)
      ldi   r18, 30
      mul   r17, r18
      add   r28, r0
      adc   r29, r1

      ldi   r19, 0                     ; r19 = loop counter

      cbi   _SFR_IO_ADDR(PORTA), PINA_LATCH

.loop:
      ldi   r18, 0                     ; r18 = red < frame ? 0 : _BV(PINA_DATA)
		ld		r0, Y+                     ; r0 = red
      sub   r0, r16
      sbrs  r0, 7
      ldi   r18, _BV(PINA_DATA)

		out   _SFR_IO_ADDR(PORTA), r18
      sbi   _SFR_IO_ADDR(PORTA), PINA_CLOCK
      cbi   _SFR_IO_ADDR(PORTA), PINA_CLOCK

      ldi   r18, 0                     ; r18 = green < frame ? 0 : _BV(PINA_DATA)
      ld    r0, Y+                     ; r0 = green
      sub   r0, r16
      sbrs  r0, 7
      ldi   r18, _BV(PINA_DATA)

		out   _SFR_IO_ADDR(PORTA), r18
      sbi   _SFR_IO_ADDR(PORTA), PINA_CLOCK
      cbi   _SFR_IO_ADDR(PORTA), PINA_CLOCK

      ldi   r18, 0                      ; r18 = counter == row ? 0 : _BV(PINA_DATA)
      cpse  r19, r17
      ldi   r18, _BV(PINA_DATA)

		out   _SFR_IO_ADDR(PORTA), r18
      sbi   _SFR_IO_ADDR(PORTA), PINA_CLOCK
      cbi   _SFR_IO_ADDR(PORTA), PINA_CLOCK

      ldi   r18, 0                     ; r18 = blue < frame ? 0 : _BV(PINA_DATA)
      ld    r0, Y+                     ; r0 = blue
      sub   r0, r16
      sbrs  r0, 7
      ldi   r18, _BV(PINA_DATA)

		out   _SFR_IO_ADDR(PORTA), r18
      sbi   _SFR_IO_ADDR(PORTA), PINA_CLOCK
      cbi   _SFR_IO_ADDR(PORTA), PINA_CLOCK

      inc   r19
      cpi   r19, 10
      brmi  .loop

      sbi   _SFR_IO_ADDR(PORTA), PINA_LATCH

      inc   r16
      cpi   r16, 16
      brmi  .out
      ldi   r16, 1

      inc   r17
      cpi   r17, 10
      brmi  .out
      ldi   r17, 0

.out:
		sts   display_frame, r16         ; r16 = frame number
      sts   display_row, r17           ; r17 = row number

      out   _SFR_IO_ADDR(SREG), r20

      pop   r29
      pop   r28
      pop   r20
      pop   r19
      pop   r18
      pop   r17
      pop   r16
      pop   r1
		pop   r0

      reti
